package controller;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import model.ComplexPredicate;
import model.WordPC;

import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;

/**
 * 
 * Class to control of use of complex predicates
 * @author gonta
 *
 */
public class ComplexPredicateController {
	
	private static ComplexPredicateController instance;
	
	static String COMPLEX_PREDICATE_NAMEFILE = ""; 
	
	Map<String,ComplexPredicate> complex_predicates;
	
	/**
	 * Constructor 
	 */
	private ComplexPredicateController() {
		complex_predicates = new HashMap<String,ComplexPredicate>();
	}	
	
	/**
	 * Method to retrieve the complex predicates 
	 * @return
	 */
	public Map<String, ComplexPredicate> get_complex_predicates() {
		return complex_predicates;
	}

	/**
	 * Method to load the list of complex predicates in memory
	 */
	public void loadComplexPredicates() {
		readConfig();		
				
		if ( !COMPLEX_PREDICATE_NAMEFILE.equals( "" ) ) {
			System.out.println( "Carregando o arquivo de predicados complexos..." );
			FileInputStream file;
			try {
				file = new FileInputStream(new File(COMPLEX_PREDICATE_NAMEFILE));			
				
				HSSFWorkbook workbook = new HSSFWorkbook(file);				 
				HSSFSheet sheet = workbook.getSheetAt(0);
				
				for (int i = 1; i < sheet.getLastRowNum(); i++) {
					HSSFRow row =  sheet.getRow(i);	
					if( row.getCell(0) != null && row.getCell(0).getStringCellValue().equals("i"))
						addComplexPredicates( complex_predicates, row.getCell( 1 ).getStringCellValue().trim(), "i" );
					else
						addComplexPredicates( complex_predicates, row.getCell( 1 ).getStringCellValue().trim(), "ni" );
				}
				
				System.out.println( "Arquivo de predicados complexos carregado." );
			} catch (FileNotFoundException e) {
				System.err.println( "Houve um erro durante a carga do arquivo." );
				e.printStackTrace();
			} catch (IOException e) {
				System.err.println( "Houve um erro durante a carga do arquivo." );
				e.printStackTrace();
			}			
		}	
	}
	

	/**
	 * Method for adding complex predicates to the list of predicate complex 
	 * @param complexPredicates List of complex predicates
	 * @param expression expression to be inserted into the list of complex predicates
	 * @param type i(idiomatic) ni(no idiomatic)
	 */
	private static void addComplexPredicates( Map<String,ComplexPredicate> complexPredicates, String expression, String type ) {
		int pos  = expression.indexOf( " " );
		String verb = expression.substring( 0, pos ).trim();
		
		String tokens [] = expression.substring( pos + 1 ).trim().split( " " );
		ComplexPredicate complexPredicate = null;		
		if( complexPredicates.containsKey( verb ) ) {
			complexPredicate = complexPredicates.get( verb );	
			WordPC dadAux = complexPredicate.root;
			for (String token : tokens) {
				dadAux = dadAux.addChild(new WordPC(dadAux, token, type));				
			}
		} else {
			WordPC root = new WordPC(null, verb);			
			WordPC rootAux = root;
			for (String token : tokens) {
				rootAux = rootAux.addChild(new WordPC(rootAux, token, type));				
			}			
			complexPredicate = new ComplexPredicate( root );
			complexPredicates.put( verb , complexPredicate );
		}

	}


	/**
	 * Read the configuration file to set the path of the complex predicate file
	 */
	private static void readConfig() {
		System.out.println( "Lendo o arquivo de configuração de predicados complexos..." );
		File archivo = null;
	    FileReader fr = null;
	    BufferedReader br = null;
	    String dirCP = "complex_predicate_dir";
	    
	    try {
	         archivo = new File ( "config" );
	         fr = new FileReader ( archivo );
	         br = new BufferedReader( fr );
	 
	         String line;
	         while( ( line=br.readLine() ) != null ) {     			 
	        	int firstOcorrence	 = line.indexOf( ":" );
	        	if( firstOcorrence != -1 ) {
		        	String key = line.substring(0, firstOcorrence).trim();
		        	String value = line.substring(firstOcorrence+1).trim();		        	
		        	if ( key.equals( dirCP ) )
		        		if ( value != null && !value.trim().equals( "" ) )
		        			 COMPLEX_PREDICATE_NAMEFILE = value;	        			
		        		else
		        			 System.err.println("Não existe o arquivo da variável \"complex_predicate_dir\" ou endereço errado.");   
	        	}
      			 
	         }       
	      }
	      catch(Exception e){
	         e.printStackTrace();
	      }		
		}
	
	/**
	 * Method to get the instance of controller (singleton)
	 * @return instance of class
	 */
    public static ComplexPredicateController getInstance(){
        if (instance == null) {
            instance = new ComplexPredicateController();
        }
        return instance;
  }	
	
	
	public static void main ( String [] args ) {
		ComplexPredicateController extra = new ComplexPredicateController();
		extra.loadComplexPredicates();
	}
	

}
