package model;

import java.util.ArrayList;
import java.util.List;

/**
 * This class represents a word that belong to a complex predicate
 * @author gonta
 *
 */
public class WordPC {
	
    private WordPC dad;
    private String value;
    private String type;
	private List<WordPC> children;
     
	/**
	 * Constructor
	 * @param dad 
	 * @param value
	 */
    public WordPC(WordPC dad, String value) {
        this.dad = dad;
        this.value = value;
        children = new ArrayList<WordPC>();
        type = "";
    }
    
    /**
     * Constructor
     * @param dad
     * @param value
     * @param type
     */
    public WordPC(WordPC dad, String value, String type) {
        this.dad = dad;
        this.value = value;
        children = new ArrayList<WordPC>();
        this.type = type;
    }
    
    public void setValor(String value) {
        this.value = value;
    }
    
    public String getValue() {
        return value;
    }
    
    /**
     * This method adds a part of the complex predicate to complex predicate
     * @param child part of the complex predicate to be added
     * @return
     */
    public WordPC addChild(WordPC child) {
    	for (WordPC aux : this.children) {
			if( aux.equals( child ) )
				return aux;
		}	
        children.add( child );
		return children.get( children.size()-1 );
    }   
    
    public List<WordPC> getChildren() {
        return this.children;
    }
    
    public WordPC getChild( String child ) {
    	if( this.children.size() == 0 ) //if doesn't have children is because it's a leaf
    		return new WordPC(null, "-");
    	
    	for ( WordPC word : this.children ) {
			if( word.getValue().equals( child ) )
				return word;
		}
    	return null;
    }
    
    public boolean isDad() {
        return !children.isEmpty();
    }
    
    public WordPC getDad() {
        return this.dad;
    }
    
    /**
     * Convert the complex predicate to a string
     */
    @Override
	public String toString() {    
        StringBuilder bString = new StringBuilder();
        bString.append(this.getValue());
        if (this.isDad()) {
            bString.append("(");
            for (WordPC n: this.getChildren()) {
                bString.append(n.toString());
                bString.append(", ");
            }
            bString.delete(bString.length()-2, bString.length());
            bString.append(")");
        }
        return bString.toString();
    }
    
    public boolean equals( WordPC wordAux ) {
    	if( wordAux.getValue().equals( this.value ) )
    		return true;
    	else
    		return false;    	
    }
    

    public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

    
}
