package view;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.swing.WindowConstants;

import model.ontology.Ontology;
import nlputil.Security;

import org.jdom.JDOMException;

import controller.Annotation;
import controller.ComplexPredicateController;
import controller.MainController;

/**
 * This class recieves all of visual components
 * 
 * @author fernando
 *
 */
public class Gui extends JFrame{
	
	//Visual Components
	private TextView text_view;
	private CommentView comment_view;
	private Translates translates;
	private SynsetView synset_view;

	//Application controllers
	private Annotation annotation;
	private MainController controller;
	
	private AnnotersView annoters_view;
	private LimitAnnotationView limitAnnotation_view;
	
	public Gui(){
		super("NASP++ - Anotador de Sentidos para o Português");
		
		setDefaultCloseOperation( WindowConstants.DO_NOTHING_ON_CLOSE );
		setSize(1000, 700);		
		
		setContentPane( new JPanel() );
		getContentPane().setLayout( new GridLayout( 0, 1 ) );
		
		JSplitPane left_panel = new JSplitPane( JSplitPane.VERTICAL_SPLIT );

		text_view = new TextView();
		text_view.setPreferredSize(new Dimension(text_view.getPreferredSize().width,300));
		
		left_panel.add( text_view );
		left_panel.add( createCommentView() );
		
		
		JSplitPane left2_panel = new JSplitPane( JSplitPane.VERTICAL_SPLIT );
		left2_panel.add(left_panel);
		left2_panel.add( createTranslateView() );
		
		JSplitPane central_panel = new JSplitPane( JSplitPane.HORIZONTAL_SPLIT );
		
		central_panel.add( left2_panel );
		central_panel.add(createSynsetView() );
		add( central_panel );
		createMenu();		
		

		limitAnnotation_view = new LimitAnnotationView( this );
		annoters_view = new AnnotersView( this );
		
		annotation = new Annotation(translates, synset_view , text_view, annoters_view, comment_view, limitAnnotation_view);
		text_view.setController( annotation );
		translates.setController( annotation );
		synset_view.setController( annotation );
		comment_view.setController ( annotation );
		limitAnnotation_view.setController ( annotation ); 
		
		
		try {
			controller = new MainController( this );
			
		} catch (IOException e) {		
			e.printStackTrace();
		}

		addFrameEvents();
		
		show();
		annoters_view.show();

		left_panel.setDividerLocation( 0.70 );
		left2_panel.setDividerLocation( 0.65 );
		central_panel.setDividerLocation( 0.5 );
		
	}

	@Override
	public void resize(int width, int height) {	
		super.resize(width, height);		
		System.out.println("NASP++ - Anotador de Sentidos para o Português...");
	}
	
	/**
	 * This method adds events to the UI 
	 */
	private void addFrameEvents() {
		
		addWindowListener( new WindowAdapter() {
			@Override
			public void windowClosing( WindowEvent e ) {
				
				if( !controller.isOpen() ){
					dispose();
					return;
				}
							
				String options[] = {"Sim", "Não" ,"Cancelar"};
				int op = JOptionPane.showOptionDialog(
						Gui.this, 
						"Deseja salvar a anotação em formato txt?\nCancelar para voltar à anotação!",
						"Salvar/Sair",
						JOptionPane.YES_NO_CANCEL_OPTION, 
						JOptionPane.QUESTION_MESSAGE,
						null, 
						options,
						options[ 0 ]);
				
				if ( op != JOptionPane.CANCEL_OPTION ){
					
					annotation.saveAnnotation( true );
					Ontology grafo = Ontology.getInstance();
					String nameFile = annotation.getName_file();
					if( !nameFile.equals("") ) {
						grafo.saveAll(new File(nameFile));	
					}
					
				if ( op == JOptionPane.YES_OPTION ){
						controller.save( text_view );
						dispose();
					}
					
				if( op == JOptionPane.NO_OPTION )
						dispose();
				}	
			}
			
		});
	}

	/**
	 * This method creates the application menu
	 */
	private void createMenu() {
		
		final JMenuBar menu_bar = new JMenuBar();
		
		//Criando os menus
		final JMenu file = new JMenu("Arquivo");
		final JMenu annotation = new JMenu("Anotação");
		
		JMenuItem open = new JMenuItem("Abrir Arquivo(s)");
		open.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				new SwingWorker(){
					@Override
					protected Object doInBackground() throws Exception {
						try{	
							if((Annotation.isUpdated && saveAndClose()) || (!Annotation.isUpdated) ) { 
									file.setEnabled( false );
									annotation.setEnabled( false );
									translates.init();
									synset_view.init();
									controller.openFile( text_view, annoters_view, limitAnnotation_view );		
								}
													
						}catch( IllegalArgumentException ex ){							
							errorPrint( ex.getMessage() );
							file.setEnabled( true );
							annotation.setEnabled( true );
						}						
						return null;
					}
					
					
					@Override
					protected void done() {					
						file.setEnabled( true );
						annotation.setEnabled( true );
					};
				}.execute();
				
			}
		});
		file.add( open );
		
		JMenuItem open_annotation = new JMenuItem("Abrir Anotação");
		open_annotation.addActionListener( new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				
				try{
					if((Annotation.isUpdated && saveAndClose()) || (!Annotation.isUpdated) ) {
							controller.openAnnotation( text_view , annoters_view, limitAnnotation_view, false );		
							translates.init();
							synset_view.init();
						}											
				
				}catch( IllegalArgumentException ex ){
				errorPrint( ex.getMessage() );
				
				} catch (IOException e) {				
					e.printStackTrace();
				
				} catch (JDOMException e) {				
					e.printStackTrace();
				}						
				menu_bar.setEnabled( true );
				
			}
		});
		file.add( open_annotation );
		
		JMenuItem limits = new JMenuItem("Limite de palavras");
		limits.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent e) {
				limitAnnotation_view.show();
			}
		});
		annotation.add( limits );
		
		JMenuItem save = new JMenuItem("Salvar Anotação em Texto");
		save.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent e) {
				controller.save( text_view );
			}
		});
		annotation.add( save );
		
		JMenuItem annoters = new JMenuItem("Anotadores");
		annoters.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent e) {
				annoters_view.show();
			}
		});
		annotation.add( annoters );
		
		//Criando o menu Ajuda
		JMenu help = new JMenu("Ajuda");
		
		JMenuItem about = new JMenuItem("Sobre");
		about.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent e) {
				
				String msg = "<html>" +
						"Ferramenta desenvolvida no <b>Núcleo Interinstitucional de Lingüística Computacional (NILC)</b>,<br>" +
						"no intuito de auxiliar anotação de córpus na tarefa de Desambiguação Lexical de Sentido<br />" +
						"para o Português do Brasil.<br /><br />" +
						"Desenvolvido por:<br />" +
						"	(Mestrando) Fernando Antônio Asevedo Nóbrega<br />" +
						"	(Mestrando) Marco Antonio Sobrevilla Cabezudo<br />" +
						"	(Orientador) Prof. Dr. Thiago Alexandre Salgueiro Pardo" +
						"<br /><br />ICMC -- USP<br />2013";
				
				JOptionPane.showMessageDialog( null, msg, "Sobre", JOptionPane.INFORMATION_MESSAGE );
			}
		});
		
		help.add( about );
		
		JMenuItem help_view = new JMenuItem("Ajuda");
		help.add( help_view );
		
		menu_bar.add( file );	
		menu_bar.add( annotation );
		menu_bar.add( help );
		
		setJMenuBar( menu_bar );
	}

	/**
	 * Method invoked when we're going to close the application.
	 * This method saves (or doesn't) and closes the application  
	 * @return
	 */
	private boolean saveAndClose() {
		String options[] = {"Sim", "Não" ,"Cancelar"};
		int op = JOptionPane.showOptionDialog(
				Gui.this, 
				"Deseja salvar a anotação?\nCancelar para voltar à anotação!",
				"Salvar/Sair",
				JOptionPane.YES_NO_CANCEL_OPTION, 
				JOptionPane.QUESTION_MESSAGE,
				null, 
				options,
				options[ 0 ]);
	
		if ( op != JOptionPane.CANCEL_OPTION ){
		
			if ( op == JOptionPane.YES_OPTION ){
				controller.save( text_view );				
				annotation.saveAnnotation( true );
				Ontology grafo = Ontology.getInstance();
				String nameFile = annotation.getName_file();
				if( !nameFile.equals("") ) {
					grafo.saveAll(new File(nameFile));	
				}
				Ontology.restart();
			}

			Annotation.isUpdated = false;
			return true;
		}
		return false;
	}


	/**
	 * This method shows in the screen error messages 
	 * @param message message to be showed
	 */
	private void errorPrint(String message) {
		
		JOptionPane.showMessageDialog(this, message, "Erro", JOptionPane.ERROR_MESSAGE);
	}

	/**
	 * This method creates the view of translation
	 * @return
	 */
	private JPanel createTranslateView(){
		
		JPanel painel = new JPanel();
		painel.setLayout( new BorderLayout() );	
		
		translates = new Translates();
		translates.setController( annotation );
		
		painel.add( translates, BorderLayout.CENTER );
		painel.setPreferredSize(new Dimension(text_view.getPreferredSize().width,200));
		return painel;		
	}
	
	/**
	 * This method creates the components for the annotation of senses
	 */
	private JPanel createSynsetView(){
		JPanel painel = new JPanel();
		painel.setLayout( new BorderLayout() );		

		synset_view = new SynsetView();	
		painel.add( synset_view , BorderLayout.CENTER );
		painel.setBorder( BorderFactory.createEmptyBorder(5, 5, 5, 5));

		return painel;
	}
	
	/**
	 * This method creates the components for the setting of comments
	 */
	private JPanel createCommentView() {
		comment_view = new CommentView( this );	
		return comment_view;
	}
	
	public static void main( String a[] ){

		System.setProperty("file.encoding", "UTF-8");		
		ComplexPredicateController.getInstance().loadComplexPredicates();
		
		try {
			Security.read();
			System.setProperty("wordnet.database.dir", Security.getInfo("wordnet_dir") );
			System.setProperty("treetagger.home", Security.getInfo("wordnet_dir") );
			System.setProperty("treetagger.home", "lib"+ File.separator + "treetagger"  );
			SwingUtilities.invokeLater(new Runnable() {
			      @Override
				public void run() {
			        new Gui().setVisible(true);
			      }
			 });

		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}
}

