package view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import model.WordNet;
import view.components.SynsetCellRender;
import controller.Annotation;
import edu.smu.tspell.wordnet.Synset;

/**
 * This class represents the UI component responsible for the selecting of synsets of the words
 * @author gonta
 *
 */
public class SynsetView extends JPanel{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private JList list;
	private JButton select_synset;
	private JButton remove_synset;
	private JCheckBox view_hype;
	private JCheckBox view_hypo;
	private JLabel error_msg;
	private DefaultListModel list_model;
	private SynsetCellRender cell_render;
	
	/**
	 * Constructor
	 */
	public SynsetView(){
		super();	
		setBorder( new TitledBorder("2 - Escolha o Synset")) ;
		
		addComponents();
	}

	/**
	 * This method add components to the panel
	 */
	private void addComponents() {
	
		setLayout( new BorderLayout() );
		
		JPanel view_buttons = new JPanel( new FlowLayout( FlowLayout.LEFT));
		
		view_hype = new JCheckBox("Ver Hiperônimos");
		view_hype.setToolTipText("Clique para ativar ou desativar a visualização dos Hiperônimos dos synsets");
		view_hype.addActionListener( new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent evt) {				
				cell_render.setViewHypernyms( view_hype.isSelected() );
				
				for( int i=0; i < list_model.size(); i++)
					list_model.set( i, list_model.get( i ) );
			}
		});
		view_buttons.add( view_hype );

		view_hypo = new JCheckBox("Ver Hipônimos");
		view_hypo.setToolTipText("Clique para ativar ou desativar a visualização dos Hipônimos dos synsets");
		view_hypo.addActionListener( new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent evt) {
				cell_render.setViewtHyponyms( view_hypo.isSelected() );
				for( int i=0; i < list_model.size(); i++)
					list_model.set( i, list_model.get( i ) );
			}
		});
		view_buttons.add( view_hypo );
		
		add( view_buttons, BorderLayout.NORTH );
		
		JPanel main_panel = new JPanel();
		main_panel.setLayout( new BoxLayout( main_panel, BoxLayout.Y_AXIS ));
		main_panel.setBorder( BorderFactory.createEmptyBorder(5, 5, 5, 5));
		
		main_panel.add( new JLabel("Synsets:"));
		main_panel.add(  Box.createRigidArea( new Dimension(0, 5)));
		
		error_msg = new JLabel("Não foi encontrado nenhum synset");
		error_msg.setForeground( Color.RED );
		
		main_panel.add( error_msg );
		error_msg.setVisible( false );		
		main_panel.add(  Box.createRigidArea( new Dimension(0, 5)));

		list = new JList();
		list.setLayoutOrientation( JList.VERTICAL );
		list.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		//modelo da lista
		list_model = new DefaultListModel();
		list.setModel( list_model );
		cell_render = new SynsetCellRender();
		list.setCellRenderer(cell_render );

		JScrollPane list_panel = new JScrollPane(list);		
		list_panel.setAlignmentX(LEFT_ALIGNMENT);
		main_panel.add( list_panel );
		main_panel.add( Box.createRigidArea( new Dimension(0, 5)));
			
		add( main_panel, BorderLayout.CENTER );
		
		JPanel annotation_buttons = new JPanel( new FlowLayout( FlowLayout.LEFT ));
		
		select_synset = new JButton("Selecionar Synset");
		annotation_buttons.add( select_synset );
				
		remove_synset = new JButton("Remover anotação");
		annotation_buttons.add( remove_synset );
		
		add( annotation_buttons, BorderLayout.SOUTH );
		
		changeActives( false, false);
	}

	/**
	 * This method add the annotation to the view, for handling events 
	 * of the annotation
	 * @param annotation 
	 */
	public void setController(final Annotation annotation) {
		
		select_synset.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent ev) {
				note(annotation);
			}
		});
		
		remove_synset.addActionListener( new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent evt) {
				annotation.removeAnnotation();			
			}
		});
		
		list.addListSelectionListener( new ListSelectionListener() {
			
			@Override
			public void valueChanged(ListSelectionEvent arg0) {
				if ( list.getSelectedIndex() < 0 )
					return;		
				
				Synset s = (Synset) list_model.get( list.getSelectedIndex() );
				
				annotation.setTranslateLabel("<html>" +						
						( list.getSelectedIndex() + 1) + "--" + 
						SynsetCellRender.formatSynonyms( s ) + "<br />" + 
						s.getDefinition() + "</html> ",
						list.getSelectedIndex() );
			}
		});
		
		list.addMouseListener(new MouseListener() {
			
			@Override
			public void mouseReleased(MouseEvent evt) {}
			
			@Override
			public void mousePressed(MouseEvent evt) {}
			
			@Override
			public void mouseExited(MouseEvent evt) {}
			
			@Override
			public void mouseEntered(MouseEvent evt) {}
			
			@Override
			public void mouseClicked(MouseEvent evt) {				
				if( evt.getClickCount() == 2 ) {
					note(annotation);
				}
				if( evt.getClickCount() == 1 ) {
					Synset syn = (Synset) list_model.get(list.getSelectedIndex());
				}
			}
		});
	}
	
	/**
	 * This method set the annotation to a word token 
	 * @param annotation the object annotation that will be used
	 */
	public void note(final Annotation annotation) {
		if ( annotation == null )
			return;
		
		if( list.getSelectedIndex() == -1)
			return;
				
		Synset synsets[] = new Synset[ list_model.size() ];
		for( int i=0; i < synsets.length; i++)
			synsets[ i ] = (Synset) list_model.get( i );
	
		boolean res = annotation.note( list.getSelectedIndex(), synsets ) ;
		if(!res)
			JOptionPane.showMessageDialog( this, "A palavra não pode ser anotada porque não pertence à classe gramatical escolhida", 
					"Erro na anotação!",
					JOptionPane.ERROR_MESSAGE );
			
	}

	/**
	 * Search the synsets that belong to the word specified and put into the list
	 * @param word word searched 
	 * @param morphoTag POS of word searched
	 */
	public void search(String word, String morphoTag) {
		
		clear();
		
		Synset synsets[] = WordNet.getSynonymsBySynsetType( word, morphoTag); 
		
		if( synsets.length == 0 )
			error_msg.setVisible( true );
		else
			changeActives( true,  remove_synset.isEnabled() );
		
		for (int i = 0; i < synsets.length; i++) 
			list_model.addElement( synsets[ i ] );
			
	}

	/**
	 * Clean the list
	 */
	public void clear() {
		error_msg.setVisible( false );
		list_model.clear();				
	}

	/**
	 * Select a synset. This method is used when We wish re-annote a word or when this word was annoted previously
	 * 
	 * @param synset synset selected
	 */
	public void setSelectedSynset(Synset synset) {
		
		for( int i=0; i < list_model.size(); i++ ){
			if ( synset.equals( list_model.get( i ) )){
				list.setSelectedIndex( i );
				break;
			}
		}
		
		changeActives( true, true);
	}
	
	/**
	 * Select a synset. This method is used when We wish re-annote a word or when this word was annoted previously
	 * @param index index of synset selected
	 */
	public void setSelectedSynset(int index) {
		
		list.setSelectedIndex( index );
	
		changeActives( true, true);
	}
	
	/**
	 * Method for changing the state of active or desactive of the components da IU 
	 * 
	 * @param annote if true, the button of "select synset" will be active
	 * @param remove if true, the button of "remove synset" will be active
	 */
	private void changeActives( boolean annote, boolean remove ){		
		this.select_synset.setEnabled( annote );
		this.remove_synset.setEnabled( remove );
	}
	
	/**
	 * Method called all of times that word was annoted. This method prepare the IU to receiving a new annotation
	 * 
	 */
	public void init( ){
		list_model.clear();
		changeActives(false, false);
	}
}
