package view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import model.Language;
import model.wsd.graphwsd.EnglishTranslator;
import nlputil.Security;
import nlputil.translate.wordreference.WordReferenceTranslate;
import view.components.TranslateCellRender;
import view.components.TranslateLabel;
import controller.Annotation;

/**
 * This class represents the component intended to translation 
 * for the words in portuguese-br Classe to the english
 * @author fernando
 *
 */
public class Translates extends JPanel{
	
	private JList list;
	private DefaultListModel list_model;	
	private JTextField new_translate;
	private boolean manual_translate;
	private JButton add_translate;	
	private JLabel wait_label;
	
	public static WordReferenceTranslate pten_translate;
	public static  WordReferenceTranslate esen_translate;
	public static  EnglishTranslator enen_translate;
	
	private int automatic_translate;
	
	private LinkedList<TranslateLabel> translates_label;
	
	/**
	 * Constructor
	 */
	public Translates(){
		super();	
		setBorder( new TitledBorder("1 - Escolha a tradução")) ;
		
		translates_label = new LinkedList<TranslateLabel>();
		
		addComponents();
		
		pten_translate = new WordReferenceTranslate( Security.getInfo("wordreference_api_pten"), "pten");
		esen_translate = new WordReferenceTranslate( Security.getInfo("wordreference_api_esen"), "esen");
		enen_translate = new EnglishTranslator();
		
		changeActives( false );
	}

	/**
	 * Add components to the screen
	 */
	private void addComponents() {
		
		JPanel p1 = new JPanel();
		p1.setLayout( new BoxLayout( p1, BoxLayout.PAGE_AXIS ));
		p1.setBorder( BorderFactory.createEmptyBorder(0, 5, 0, 5)); 
		p1.setAlignmentX(RIGHT_ALIGNMENT);
		
		p1.add( new JLabel("Possíveis traduções:"));
		
		wait_label = new JLabel();
		wait_label.setVisible( false );
		p1.add( wait_label );
		
		p1.add( Box.createRigidArea( new Dimension(0, 5))); 
		
		list = new JList();
		list.setLayoutOrientation( JList.VERTICAL_WRAP );
		list.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		list.setCellRenderer( new TranslateCellRender( translates_label ) );
		list.setVisibleRowCount( 5 );

		list_model = new DefaultListModel();
		list.setModel( list_model );
		
		JScrollPane list_panel = new JScrollPane(list); 
		list_panel.setAlignmentX(LEFT_ALIGNMENT);
		p1.add( list_panel );
		p1.add( Box.createRigidArea( new Dimension(0, 5)));		
		
		JPanel p2 = new JPanel();
		p2.setBorder( BorderFactory.createEmptyBorder(10, 5, 10, 5)); 
		p2.setLayout( new BoxLayout( p2, BoxLayout.X_AXIS ));
		p2.add( Box.createHorizontalGlue() );
		
		p2.add( new JLabel("Nova tradução:"));
		p2.add( Box.createRigidArea( new Dimension(5, 0)));

		new_translate = new JTextField(10);
		new_translate.setToolTipText("Indique uma tradução mais adequada");
		p2.add( new_translate );
		p2.add( Box.createRigidArea( new Dimension(5, 0)));

		p2.add( Box.createHorizontalGlue() );
		add_translate = new JButton("Adicionar nova tradução");
		add_translate.setToolTipText("Clique para usar a nova tradução sugerida");
		add_translate.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent ev) {	
				
				String word = new_translate.getText();
				if ( word.isEmpty() )
					return;
				
				manual_translate = true;
				automatic_translate++;
				translates_label.addFirst( new TranslateLabel( "",  -1 ) );
				list_model.add( 0 , word );				
				new_translate.setText("");
			}
		});
		p2.add( add_translate );
		
		
		setLayout( new BorderLayout() );		
		add( p1 , BorderLayout.CENTER );
		add( p2 , BorderLayout.PAGE_END );
	}

	/**
	 * Method responsible for adding events to the components
	 * of the translation
	 * 
	 * @param annotation a object that encapsulates the controller of the application
	 */
	public void setController(final Annotation annotation) {
		list.addListSelectionListener( new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				if ( list.getSelectedIndex() < 0 )
					return;
							
				String word = (String) list.getSelectedValue();
				
				if ( annotation != null){
					annotation.searchSynset( word , manual_translate );
					
					TranslateLabel selected_label =  translates_label.get( list.getSelectedIndex());
					if( selected_label.index >= 0 )
						annotation.setSelectedSynset( selected_label.index );
				}
			}
		});
		
		
	}

	/**
	 * This method recieves a word and a list of possible translates written in english
	 * @param word represents the word to be translated
	 * @param language language of the word
	 */
	public void translate( String word , Language language) {

		init();
		
		wait_label.setVisible( true );
		wait_label.setText("Traduzindo...");
		paintComponents( getGraphics() );		
		repaint();
		
		manual_translate = false;
		
		ArrayList<String> translates = null;
		try {
			
			//Verificando o idioma da palavra
			switch( language ){
				case PORTUGUESE:
					translates = pten_translate.translateWord( word );
				break;
				case SPANISH:
					translates = esen_translate.translateWord( word );
				break;
				case ENGLISH:
					translates = new ArrayList<String>(  );
					translates.add( word );
				break;
			}
			
			for (String string : translates) {
				System.out.println(string);
			}
			
			if( translates.isEmpty() ){
				wait_label.setForeground( Color.RED );
				wait_label.setText("Não foi encontrado nenhuma tradução para: " + word );
			}else{				
				for(int i=0; i < translates.size(); i++ ){
					translates_label.add( new TranslateLabel("", -1));
					list_model.addElement( translates.get( i ) );					
				}
				
				automatic_translate = 0;
				
				wait_label.setVisible( false );
			}
			
		} catch (Exception e) {
			
			wait_label.setText("Erro durante a tradução da palavra: "+ word );
			wait_label.setForeground( Color.RED );
					
			e.printStackTrace();
		
		}
		changeActives( true );
		
		repaint();
	}

	/**
	 * This method adds a translate previous. it's used for pre-annotated words and its translate possible 
	 * is listed
	 * @param word word translate added
	 */
	public void addTranslate( String word ){
		changeActives( true );
				
		translates_label.add( 0,   new TranslateLabel("", -1));
		list_model.addElement( word );
		
		automatic_translate++;
		repaint();
	}
	
	/**
	 * This method set the translates into the variables
	 * @param translates set of translates
	 */
	public void setTranslate( String translates[]){
		init();
		changeActives( true );
		
		if ( translates.length <= 0 )
			return;
	
		for( int i=0; i < translates.length; i++ ){
			translates_label.add( new TranslateLabel("", i ));
			list_model.addElement( translates[ i ] );			
		}
	}
	
	/**
	 * This method inserts the translate in the list
	 * @param translate  text to select
	 */
	public void setSelectedTranslate( String translate ){
	
		for( int i=0; i < list_model.size(); i++ )
			if( list_model.get( i ).toString().compareTo( translate) == 0){
				list.setSelectedIndex( i );
				break;
			}
				
	}
	
	/**
	 * This method obtains all of translates showed in the UI 
	 * @return
	 */
	public String[] getTranslates(){
		
		if ( list_model.size() ==0 )
			return new String[ 0 ];

		String array[];
		int index = 0;
		//Se realmente foi selecionado uma traducao manualmente
		if( list.getSelectedIndex() < automatic_translate ){
			array = new String[ list_model.size() -  automatic_translate + 1 ];
			
			array[ 0 ] = (String) list.getSelectedValue();
			index = 1;
			
		}else{
			array = new String[ list_model.size() -  automatic_translate ];
		}
		
		for(int i= automatic_translate ; i < list_model.size(); i++)
			array[ index++ ] = (String) list_model.get( i );		
		
		return array;
	}
	
	
	/**
	 * This method change the state active/desactive of UI 
	 * @param add_translate if true, allows adding new translates
	 */
	private void changeActives( boolean add_translate ){		
		this.new_translate.setEditable( add_translate );
		this.add_translate.setEnabled( add_translate );
	}
	
	/**
	 * Method inicialized every time that a word is annotated. This method
	 * prepares a interface to receive a new translate
	 */
	public void init( ){
		
		list_model.clear();
		translates_label.clear();
		
		changeActives( false );
		
		wait_label.setForeground( list.getForeground() );
		wait_label.setVisible( false );
		
		automatic_translate = 0;
	}
	
	public int getAutomaticTranslate(){
		return automatic_translate;
	}

	public boolean selectManual() {
		return list.getSelectedIndex() < automatic_translate;
	}

	public void setTranslateLabel(String label, int index ) {		
		this.translates_label.set( list.getSelectedIndex(),  new TranslateLabel(
				"<html>" + list.getSelectedValue() + "<br />" + label, index ) );		
	}
	
	public static void main (String [] args) { 

		try {
			Security.read();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Translates t = new Translates();
		t.translate("abafar", Language.PORTUGUESE);
	}

}
