package view.components;

import java.awt.Color;
import java.awt.Font;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.LinkedList;

import javax.swing.JLabel;
import javax.swing.UIManager;
import javax.swing.border.LineBorder;

import model.Comment;
import model.Language;
import model.MorphoTags;
import model.StateAnnotation;
import edu.smu.tspell.wordnet.Synset;

/**
 * That class represents the tokens that will be showed to the user.
 * Those tokens can have or haven't possible annotations, depending of its type
 * 
 * @author fernando
 *
 */
public class Tokens extends JLabel implements Serializable{
	
	/* ***************************
	***** Attributes of token  ****
	*****************************/
	
	private String lemma;
	
	private String tag; 
	
	private MorphoTags morpho_tag;
	
	private MorphoTags morpho_tag_pos;
	
	private int annoted_state; 
	private StateAnnotation state_annoted;
	
	private boolean manual_translate; 	
	
	private LinkedList<Synset> synsets_set;
	private String[] translates_set;
	private String translate;
	private Synset annoted_synset;
	
	private ArrayList<Tokens> linked_tokens;
	
	private Language language;
	
	/* Comment of the token */
	private model.Comment comment;
	private String commentObs = "";
	
	private String partOfComplexPredicate = "";
	
	/**
	 * Constructor of the class
	 * @param word {@link String} store a lexical item
	 * @param annoted_state {@link StateAnnotation} represents the initial state of the token annotation
	 * @param tag {@link String} POS-Tag attributed to token during a stage of pre-process
	 * @param tag {@link MorphoTags} tag
	 * @param language {@link Language} represents the language text
	 */
	public Tokens( String word,
					StateAnnotation annoted_state,
					String tag,
					MorphoTags morpho_tag,
					Language language){
		super( word );		
		setOpaque( true );
		
		this.state_annoted = annoted_state;
		this.tag = tag;
		this.translate = null;
		this.annoted_synset = null;
		this.language = language;
		this.morpho_tag = morpho_tag;
		this.morpho_tag_pos = morpho_tag;
		lemma = word;
		comment = model.Comment.NO_COMMENTS;
		changeType( this.state_annoted );
		
		if ( state_annoted != StateAnnotation.NO_ANNOTE )
			linked_tokens = new ArrayList<Tokens>( );
	}

	/**
	 * This method sets the state of annotation for a token
	 * @param annotted_state state of the annotation
	 */
	public void setType( StateAnnotation annotted_state ){
		changeType(annotted_state);
	}
	
	
	/**
	 * This method modifies the type token
	 * @param annoted_state um valor {@link int} represents the new toke type
	 */
	public void changeType( StateAnnotation annoted_state ) {
		
		this.state_annoted = annoted_state;
		
		setBackground( Color.WHITE );
		
		if ( annoted_state == StateAnnotation.NO_ANNOTATED ){
			setToolTipText("Palavra ainda não anotada!");	
			setBorder( null );
			return;
		}
		
		if ( annoted_state == StateAnnotation.NOUN_NO_ANNOTATED ){
			setToolTipText("Substantivo ainda não anotado!");
			return;
		}
		
		if ( annoted_state == StateAnnotation.VERB_NO_ANNOTATED ){
			setToolTipText("Verbo ainda não anotado!");
			return;
		}
		
		if ( annoted_state == StateAnnotation.VERB_AUX_NO_ANNOTATED ){
			setToolTipText("Verbo Auxiliar não anotado!");
			return;
		}
		
		if ( annoted_state == StateAnnotation.ANNOTATED ){
			if( this.morpho_tag.compareTo(MorphoTags.VERB) == 0 )
				setToolTipText("Verbo anotado!");
			else if( this.morpho_tag.compareTo(MorphoTags.NOUN) == 0 )
				setToolTipText("Substantivo anotado!");
			else
				setToolTipText("Palavra anotada!");
			return;
		}
		
		if ( annoted_state == StateAnnotation.PREV_ANNOTATED ){
			setToolTipText("Palavra previamente anotada!");
			return;
		}
				
	}
	
	/**
	 * This method activate the border that represents the state of the annotation
	 */
	public void activeBorder() {
		if ( state_annoted == StateAnnotation.NO_ANNOTATED )
			setBorder( null );
		
		if ( state_annoted == StateAnnotation.VERB_NO_ANNOTATED || state_annoted == StateAnnotation.NOUN_NO_ANNOTATED )
			setBorder( new LineBorder( Color.red, 2 ) );
		
		if ( state_annoted == StateAnnotation.ANNOTATED )
			setBorder( new LineBorder( Color.green ,2 ) );
		
		if ( state_annoted == StateAnnotation.PREV_ANNOTATED){
			setBackground( Color.YELLOW );
			setBorder( new LineBorder( Color.yellow, 2) );
		}
//		
		if( partOfComplexPredicate.startsWith("CP") && state_annoted == StateAnnotation.VERB_NO_ANNOTATED )
			activeBorderCP();
		
		if( partOfComplexPredicate.startsWith("CP") && state_annoted == StateAnnotation.ANNOTATED && morpho_tag == MorphoTags.VERB)
			activeBorderCP();
		
		if( partOfComplexPredicate.startsWith("CP") && state_annoted == StateAnnotation.NO_ANNOTATED )
			desactiveBorderCP();	
	}
	
	/**
	 * This method activate the border if the token is a complex predicate
	 */
	public void activeBorderCP() {
		setFont(new Font( "Courier", Font.BOLD + Font.ITALIC, 12 ));
		if(partOfComplexPredicate.equals("CPI"))
			setForeground (Color.gray);
		else
			setForeground(Color.cyan);
	}

	/**
	 * This method desactivate the border for a complex predicate
	 */
	public void desactiveBorderCP() {
		setFont( UIManager.getFont( "Label.font" ) );
		setForeground ( UIManager.getColor("Label.color"));
	}


	/**
	 * This method desactivate the border for all types of verb
	 */
	public void desactiveBorder() {
		setBorder( null );
		setBackground( Color.WHITE );	
		
		if (partOfComplexPredicate.startsWith("CP") && (state_annoted == StateAnnotation.NO_ANNOTATED || state_annoted == StateAnnotation.VERB_NO_ANNOTATED))
			desactiveBorderCP(); 
		
		if (partOfComplexPredicate.startsWith("CP") && state_annoted == StateAnnotation.ANNOTATED && morpho_tag == MorphoTags.VERB )
			desactiveBorderCP(); 
	}
	
	
	/**
	 * 
	 * @return
	 */
	public StateAnnotation getAnnotedState(){
		return this.state_annoted;
	}
	
	/**
	 * 
	 * @return
	 */
	public boolean isAnnotedState(){
		if (this.state_annoted.equals(StateAnnotation.ANNOTATED))
			return true;
		return false;
	}

	/**
	 * This method adds the annotation to a word
	 * 
	 * @param manual_translate {@link boolean} true(word translate manually)
	 * @param translate {@link String} represent a translate for english
	 * @param selected_synset {@link int} represents the synset selected in the list of possible
	 * @param synset a vector including all of possible synsets found for a word
	 */
	public void setAnnotation( boolean manual_translate, String translate, Synset s, String[] translate_set, LinkedList<Synset> synsets_set ){
		
		this.manual_translate = manual_translate;
		this.translate = translate;
		this.translates_set = translate_set.clone();
		this.annoted_synset = s;		
		this.synsets_set = (LinkedList<Synset>) synsets_set.clone();
				
		changeType( StateAnnotation.ANNOTATED );
		activeBorder();
		
		if ( linked_tokens != null )
			for( Tokens t : linked_tokens )
				t.setPreAnnotation( manual_translate, translate,  s, translate_set, synsets_set, true );
	}
	

	/**
	 * This method adds annotation to the token, but It's used 
	 * when the annotation have a verb comment, i.e., Auxiliar verb
	 */
	public void setCommentAnnotation(){
		changeType( StateAnnotation.ANNOTATED );
		this.manual_translate = false;
		this.translate = null;
		this.translates_set = null;
		this.annoted_synset = null;
		this.synsets_set = null;
		activeBorder();		
	}
	
	/**
	 * This method adds the previous annotation to the word, if this has not yet been annotated
	 * @param manual_translate  true (word translate manually)
	 * @param translate represents the translate for english
	 * @param s the selected synset
	 * @param translate_set set of possible translations
	 * @param synsets_set set of possible synsets
	 * @param first if true, verifies the related words 
	 */
	public void setPreAnnotation( boolean manual_translate, 
			String translate, Synset s, String[] translate_set, LinkedList<Synset> synsets_set, boolean first) {
		
		if (this.state_annoted != StateAnnotation.ANNOTATED  ){		
			this.manual_translate = manual_translate;
			this.translate = translate;
			this.translates_set = translate_set.clone();
			this.annoted_synset = s;		
			this.synsets_set = (LinkedList<Synset>) synsets_set.clone();

			changeType( StateAnnotation.PREV_ANNOTATED );
			activeBorder();
		}
		
		if ( first )
			if ( linked_tokens != null )
				for( int i=0; i < linked_tokens.size(); i++ )						
					linked_tokens.get( i ).setPreAnnotation( manual_translate, translate,  s, translate_set, synsets_set, false );
					
	}
		
	/**
	 * Remove the previous annotation of the word, if it is not yet annotated
	 * 
	 */
	public void removePreAnnotation() {
				
		if (this.state_annoted != StateAnnotation.ANNOTATED  ){	
			if( this.morpho_tag.equals( MorphoTags.NOUN ) )
				changeType( StateAnnotation.NOUN_NO_ANNOTATED );
			else if( this.morpho_tag.equals( MorphoTags.VERB ) )
				changeType( StateAnnotation.VERB_NO_ANNOTATED );
			else
				changeType( StateAnnotation.NO_ANNOTATED );
			
			activeBorder();
			this.manual_translate = false;
			this.translate = null;
			this.translates_set = null;
			this.annoted_synset = null;
			this.synsets_set = null;
		}
	}
	
	/**
	 * @return the formatted word for annotation. if the word has note annotation
	 * the word is returned alone, in other case, returns a pattern of annotation
	 */
	public String write(){
		
		if( annoted_synset == null )
			return getText();
		
		StringBuilder output = new StringBuilder();
		output.append( getText() );
		output.append( "<" );
		output.append( translate );
		output.append(",");
		output.append( annoted_synset.toString().split("]")[ 0 ] );
		output.append( "]>" );
				
		return output.toString();			
	}

	/**
	 * This method gets the translations for a word
	 * @return {@link String} represents a translation used for this word
	 */
	public String getTranslate() {
		return translate;
	}

	/**
	 * @return true, true if has a selected synset
	 */
	public boolean isAnnoted() {
		return (annoted_synset != null );
	}

	
	public Synset getSynset(){
		
		if( annoted_synset == null )
			return null;
		
		return annoted_synset;
	}
	

	public void addLink(Tokens link_token ) {
		linked_tokens.add( link_token );
	}

	/**
	 * This method remove the annotation for a annotated and pre-annotated word
	 */
	public void removeAnnotation() {
		if( this.morpho_tag.equals( MorphoTags.NOUN ) )
			changeType( StateAnnotation.NOUN_NO_ANNOTATED );
		else if( this.morpho_tag.equals( MorphoTags.VERB ) )
			changeType( StateAnnotation.VERB_NO_ANNOTATED );
		else
			changeType( StateAnnotation.NO_ANNOTATED );
		
		activeBorder();
		this.manual_translate = false;
		this.translate = null;
		this.translates_set = null;
		this.annoted_synset = null;
		this.synsets_set = null;
		
		if ( linked_tokens != null )
			for( Tokens t : linked_tokens )
				t.removePreAnnotation();
	}
	
	public boolean getmanual_translate(){
		return this.manual_translate;
	}

	public String[] getTtranslateSet() {
		return translates_set;
	}

	public LinkedList<Synset> getSynsetSet() {
		return synsets_set;
	}

	public String getTag(){
		return this.tag;
	}

	
	public MorphoTags getMorphoTag() {
		return this.morpho_tag;
	}
	
	public void setMorphoTag( MorphoTags morpho_tag ){
		this.morpho_tag = morpho_tag;
	}
	
	public MorphoTags getMorphoTagPOS() {
		return this.morpho_tag_pos;
	}
	
	public void setMorphoTagPOS( MorphoTags morpho_tag ){
		this.morpho_tag_pos = morpho_tag;
	}
	
	public String getLemma() {
		return this.lemma;
	}
	
	public void setLemma( String lemma ){
		this.lemma = lemma;
	}
	
	public ArrayList<Tokens> getLinkedTokens(){
		return this.linked_tokens;
	}
	
	public Language getLanguage(){
		return this.language;
	}

	public Comment getComment() {
		return comment;
	}

	public void setComment(Comment comment) {
		this.comment = comment;
	}

	public void setCommentObs(String commentObs) {
		this.commentObs = commentObs;		
	}
	
	public String getCommentObs() {
		return commentObs;		
	}
	
	public void setTag( String tag ) {
		this.tag = tag;
	}
		
	/**
	 * This method verifies if a token is part of a complex predicate
	 * @return
	 */
	public String isPartOfComplexPredicate() {
		if(morpho_tag_pos == MorphoTags.VERB  && state_annoted == StateAnnotation.NO_ANNOTATED) {
			desactiveBorderCP();
			return "";
		}
			
		return partOfComplexPredicate;
	}

	public void setPartOfComplexPredicate(String partOfComplexPredicate) {
		this.partOfComplexPredicate = partOfComplexPredicate;
	}

	public String getPartOfComplexPredicate() {
		return partOfComplexPredicate; 
	}

}
